@@ -26,10 +26,14 @@
 #include <config.h>
 #include "gnome-vfs-uri.h"
 
-#include "gnome-vfs-private.h"
+#include "gnome-vfs-module.h"
+#include "gnome-vfs-private-utils.h"
 #include "gnome-vfs-transform.h"
-#include "gnome-vfs.h"
-#include <ctype.h>
+#include "gnome-vfs-utils.h"
+#include <glib/ghash.h>
+#include <glib/gmessages.h>
+#include <glib/gstrfuncs.h>
+#include <glib/gstring.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -140,6 +144,7 @@ split_toplevel_uri (const gchar *path, guint path_len,
 	const char *cur;
 	const char *next_delimiter;
 	char *ret;
+	char *host;
 	gboolean success;
 
 	g_assert (host_return != NULL);
@@ -233,7 +238,7 @@ split_toplevel_uri (const gchar *path, guint path_len,
 
 		port = 0;
 
-		for ( ; cur < path_end && '\0' != *cur && isdigit ((guchar) *cur); cur++) {
+		for ( ; cur < path_end && g_ascii_isdigit (*cur); cur++) {
 			port *= 10;
 			port += *cur - '0'; 
 		}
@@ -273,7 +278,9 @@ split_toplevel_uri (const gchar *path, guint path_len,
 
 done:
 	if (*host_return != NULL) {
-		g_strdown (*host_return);
+		host = g_ascii_strdown (*host_return, -1);
+		g_free (*host_return);
+		*host_return = host;
 
 	}
 
@@ -348,16 +355,18 @@ static const gchar *
 get_method_string (const gchar *substring, gchar **method_string)
 {
 	const gchar *p;
+	char *method;
 	
 	for (p = substring;
-	     isalnum ((unsigned char)*p) || *p == '+' || *p == '-' || *p == '.';
+	     g_ascii_isalnum (*p) || *p == '+' || *p == '-' || *p == '.';
 	     p++)
 		;
 
 	if (*p == ':') {
 		/* Found toplevel method specification.  */
-		*method_string = g_strndup (substring, p - substring);
-		g_strdown (*method_string);
+		method = g_strndup (substring, p - substring);
+		*method_string = g_ascii_strdown (method, -1);
+		g_free (method);
 		p++;
 	} else {
 		*method_string = g_strdup ("file");
@@ -571,9 +580,14 @@ is_uri_relative (const char *uri)
  * Compact "../" segments inside the URI
  * Remove "." at the end of the URL 
  * Leave any ".."'s at the beginning of the URI
+ 
+*/
+/*
+ * FIXME this is not the simplest or most time-efficent way
+ * to do this.  Probably a far more clear way of doing this processing
+ * is to split the path into segments, rather than doing the processing
+ * in place.
  */
-
-/* in case if you were wondering, this is probably one of the least time-efficient ways to do this*/
 static void
 remove_internal_relative_components (char *uri_current)
 {
@@ -785,23 +799,27 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 }
 
 /**
- * gnome_vfs_uri_resolve_relative:
+ * gnome_vfs_uri_resolve_relative
  * @base: The base URI.
- * @relative_reference: A string representing a possibly-relative URI reference
+ * @text_uri: A string representing a possibly relative URI reference
  * 
- * Create a new URI from @text_uri relative to @base.
+ * Create a new URI from @relative_reference, relative to @base.
  *
  * Return value: The new URI.
  **/
 GnomeVFSURI *
 gnome_vfs_uri_resolve_relative (const GnomeVFSURI *base,
-				const char *relative_reference)
+				const gchar *relative_reference)
 {
 	char *text_base;
 	char *text_new;
 	GnomeVFSURI *uri;
 
-	text_base = gnome_vfs_uri_to_string (base, 0);
+	if (base == NULL) {
+		text_base = g_strdup ("");
+	} else {
+		text_base = gnome_vfs_uri_to_string (base, 0);
+	}
 	text_new = make_full_uri_from_relative (text_base, relative_reference);
 
 	uri = gnome_vfs_uri_new (text_new);
@@ -929,11 +947,11 @@ gnome_vfs_uri_dup (const GnomeVFSURI *uri)
  **/
 GnomeVFSURI *
 gnome_vfs_uri_append_string (const GnomeVFSURI *uri,
-			     const char *uri_part_string)
+			     const gchar *uri_part_string)
 {
-	char *uri_string;
+	gchar *uri_string;
 	GnomeVFSURI *new_uri;
-	char *new_string;
+	gchar *new_string;
 	guint len;
 
 	g_return_val_if_fail (uri != NULL, NULL);
@@ -983,9 +1001,9 @@ gnome_vfs_uri_append_string (const GnomeVFSURI *uri,
  **/
 GnomeVFSURI *
 gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
-			   const char *path)
+			   const gchar *path)
 {
-	char *escaped_string;
+	gchar *escaped_string;
 	GnomeVFSURI *new_uri;
 	
 	escaped_string = gnome_vfs_escape_path_string (path);
@@ -1007,9 +1025,9 @@ gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
  **/
 GnomeVFSURI *
 gnome_vfs_uri_append_file_name (const GnomeVFSURI *uri,
-				const char *file_name)
+				const gchar *file_name)
 {
-	char *escaped_string;
+	gchar *escaped_string;
 	GnomeVFSURI *new_uri;
 	
 	escaped_string = gnome_vfs_escape_string (file_name);
@@ -1030,14 +1048,14 @@ gnome_vfs_uri_append_file_name (const GnomeVFSURI *uri,
  * 
  * Return value: A malloced printable string representing @uri.
  **/
-char *
+gchar *
 gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 			 GnomeVFSURIHideOptions hide_options)
 {
 	GString *string;
-	char *result;
+	gchar *result;
 
-	string = g_string_new(uri->method_string);
+	string = g_string_new (uri->method_string);
 	g_string_append_c (string, ':');
 
 	if (uri->parent == NULL) {
@@ -1080,7 +1098,7 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 		
 		if (top_level_uri->host_port > 0 
 			&& (hide_options & GNOME_VFS_URI_HIDE_HOST_PORT) == 0) {
-			char tmp[128];
+			gchar tmp[128];
 			sprintf (tmp, ":%d", top_level_uri->host_port);
 			g_string_append (string, tmp);
 		}
@@ -1167,7 +1185,7 @@ gnome_vfs_uri_get_parent (const GnomeVFSURI *uri)
 	g_return_val_if_fail (uri != NULL, NULL);
 
 	if (uri->text != NULL && strchr (uri->text, GNOME_VFS_URI_PATH_CHR) != NULL) {
-		char *p;
+		gchar *p;
 		guint len;
 
 		len = strlen (uri->text);
@@ -1245,7 +1263,7 @@ gnome_vfs_uri_get_toplevel (const GnomeVFSURI *uri)
  * 
  * Return value: A string representing the host name.
  **/
-const char *
+const gchar *
 gnome_vfs_uri_get_host_name (const GnomeVFSURI *uri)
 {
 	GnomeVFSToplevelURI *toplevel;
@@ -1264,7 +1282,7 @@ gnome_vfs_uri_get_host_name (const GnomeVFSURI *uri)
  *
  * Return value: A string representing the scheme
  **/
-const char *
+const gchar *
 gnome_vfs_uri_get_scheme (const GnomeVFSURI *uri)
 {
 	return uri->method_string;
@@ -1298,7 +1316,7 @@ gnome_vfs_uri_get_host_port (const GnomeVFSURI *uri)
  * 
  * Return value: A string representing the user name in @uri.
  **/
-const char *
+const gchar *
 gnome_vfs_uri_get_user_name (const GnomeVFSURI *uri)
 {
 	GnomeVFSToplevelURI *toplevel;
@@ -1317,7 +1335,7 @@ gnome_vfs_uri_get_user_name (const GnomeVFSURI *uri)
  * 
  * Return value: The password for @uri.
  **/
-const char *
+const gchar *
 gnome_vfs_uri_get_password (const GnomeVFSURI *uri)
 {
 	GnomeVFSToplevelURI *toplevel;
@@ -1337,7 +1355,7 @@ gnome_vfs_uri_get_password (const GnomeVFSURI *uri)
  **/
 void
 gnome_vfs_uri_set_host_name (GnomeVFSURI *uri,
-			     const char *host_name)
+			     const gchar *host_name)
 {
 	GnomeVFSToplevelURI *toplevel;
 
@@ -1379,7 +1397,7 @@ gnome_vfs_uri_set_host_port (GnomeVFSURI *uri,
  **/
 void
 gnome_vfs_uri_set_user_name (GnomeVFSURI *uri,
-			     const char *user_name)
+			     const gchar *user_name)
 {
 	GnomeVFSToplevelURI *toplevel;
 
@@ -1400,7 +1418,7 @@ gnome_vfs_uri_set_user_name (GnomeVFSURI *uri,
  **/
 void
 gnome_vfs_uri_set_password (GnomeVFSURI *uri,
-			    const char *password)
+			    const gchar *password)
 {
 	GnomeVFSToplevelURI *toplevel;
 
@@ -1413,7 +1431,7 @@ gnome_vfs_uri_set_password (GnomeVFSURI *uri,
 }
 
 static gboolean
-string_match (const char *a, const char *b)
+string_match (const gchar *a, const gchar *b)
 {
 	if (a == NULL || *a == '\0') {
 		return b == NULL || *b == '\0';
@@ -1578,7 +1596,7 @@ gnome_vfs_uri_is_parent (const GnomeVFSURI *possible_parent,
  * pointer points to the name store in @uri, so the name returned must not
  * be modified nor freed.
  **/
-const char *
+const gchar *
 gnome_vfs_uri_get_path (const GnomeVFSURI *uri)
 {
 	/* FIXME bugzilla.eazel.com 1472 */
@@ -1598,7 +1616,7 @@ gnome_vfs_uri_get_path (const GnomeVFSURI *uri)
  * 
  * Return value: A pointer to the fragment identifier for the uri or NULL.
  **/
-const char *
+const gchar *
 gnome_vfs_uri_get_fragment_identifier (const GnomeVFSURI *uri)
 {
 	g_return_val_if_fail (uri != NULL, NULL);
@@ -1606,41 +1624,6 @@ gnome_vfs_uri_get_fragment_identifier (const GnomeVFSURI *uri)
 	return uri->fragment_id;
 }
 
-/**
- * gnome_vfs_uri_get_basename:
- * @uri: A GnomeVFSURI
- * 
- * Retrieve base file name for @uri.
- * 
- * Return value: A pointer to the base file name in @uri.  Notice that the
- * pointer points to the name store in @uri, so the name returned must not
- * be modified nor freed.
- **/
-const char *
-gnome_vfs_uri_get_basename (const GnomeVFSURI *uri)
-{
-	/* FIXME bugzilla.eazel.com 1472: query parts of URIs aren't handled */
-	char *p;
-
-	g_return_val_if_fail (uri != NULL, NULL);
-
-	if (uri->text == NULL) {
-		return NULL;
-	}
-
-	p = strrchr (uri->text, GNOME_VFS_URI_PATH_CHR);
-	if (p == NULL) {
-		return NULL;
-	}
-
-	p++;
-	if (*p == '\0') {
-		return NULL;
-	}
-
-	return p;
-}
-
 /**
  * gnome_vfs_uri_extract_dirname:
  * @uri: A GnomeVFSURI
@@ -1652,15 +1635,16 @@ gnome_vfs_uri_get_basename (const GnomeVFSURI *uri)
  * Return value: A pointer to the newly allocated string representing the
  * parent directory.
  **/
-char *
+gchar *
 gnome_vfs_uri_extract_dirname (const GnomeVFSURI *uri)
 {
-	const char *base;
+	const gchar *base;
 
 	g_return_val_if_fail (uri != NULL, NULL);
 
-	base = gnome_vfs_uri_get_basename (uri);
-	if (base == NULL || base == uri->text) {
+	base = strrchr (uri->text, GNOME_VFS_URI_PATH_CHR);
+
+	if (base == NULL || base[1] == '\0') {
 		return g_strdup (GNOME_VFS_URI_PATH_STR);
 	}
 
@@ -1683,11 +1667,11 @@ gnome_vfs_uri_extract_dirname (const GnomeVFSURI *uri)
  * Return value: A pointer to the newly allocated string representing the
  * unescaped short form of the name.
  **/
-char *
+gchar *
 gnome_vfs_uri_extract_short_name (const GnomeVFSURI *uri)
 {
-	char *escaped_short_path_name, *short_path_name;
-	const char *host_name;
+	gchar *escaped_short_path_name, *short_path_name;
+	const gchar *host_name;
 
 	escaped_short_path_name = gnome_vfs_uri_extract_short_path_name (uri);
 	short_path_name = gnome_vfs_unescape_string (escaped_short_path_name, "/");
@@ -1723,10 +1707,10 @@ gnome_vfs_uri_extract_short_name (const GnomeVFSURI *uri)
  * Return value: A pointer to the newly allocated string representing the
  * escaped short form of the name.
  **/
-char *
+gchar *
 gnome_vfs_uri_extract_short_path_name (const GnomeVFSURI *uri)
 {
-	const char *p, *short_name_start, *short_name_end;
+	const gchar *p, *short_name_start, *short_name_end;
 
 	g_return_val_if_fail (uri != NULL, NULL);
 
@@ -1838,3 +1822,225 @@ gnome_vfs_uri_list_free (GList *list)
 {
 	g_list_free (gnome_vfs_uri_list_unref (list));
 }
+
+
+static gboolean
+is_uri_partial (const char *uri)
+{
+	const char *current;
+
+	/* RFC 2396 section 3.1 */
+	for (current = uri ; 
+		*current
+		&& 	((*current >= 'a' && *current <= 'z')
+			 || (*current >= 'A' && *current <= 'Z')
+			 || (*current >= '0' && *current <= '9')
+			 || ('-' == *current)
+			 || ('+' == *current)
+			 || ('.' == *current)) ;
+	     current++);
+
+	return  !(':' == *current);
+}
+
+/**
+ * gnome_vfs_uri_make_full_from_relative:
+ * 
+ * Returns a full URI given a full base URI, and a secondary URI which may
+ * be relative.
+ *
+ * Return value: the URI (NULL for some bad errors).
+ **/
+char *
+gnome_vfs_uri_make_full_from_relative (const char *base_uri,
+				       const char *relative_uri)
+{
+	char *result = NULL;
+
+	/* See section 5.2 in RFC 2396 */
+
+	if (base_uri == NULL && relative_uri == NULL) {
+		result = NULL;
+	} else if (base_uri == NULL) {
+		result = g_strdup (relative_uri);
+	} else if (relative_uri == NULL) {
+		result = g_strdup (base_uri);
+	} else if (!is_uri_partial (relative_uri)) {
+		result = g_strdup (relative_uri);
+	} else {
+		char *mutable_base_uri;
+		char *mutable_uri;
+
+		char *uri_current;
+		size_t base_uri_length;
+		char *separator;
+
+		mutable_base_uri = g_strdup (base_uri);
+		uri_current = mutable_uri = g_strdup (relative_uri);
+
+		/* Chew off Fragment and Query from the base_url */
+
+		separator = strrchr (mutable_base_uri, '#'); 
+
+		if (separator) {
+			*separator = '\0';
+		}
+
+		separator = strrchr (mutable_base_uri, '?');
+
+		if (separator) {
+			*separator = '\0';
+		}
+
+		if ('/' == uri_current[0] && '/' == uri_current [1]) {
+			/* Relative URI's beginning with the authority
+			 * component inherit only the scheme from their parents
+			 */
+
+			separator = strchr (mutable_base_uri, ':');
+
+			if (separator) {
+				separator[1] = '\0';
+			}			  
+		} else if ('/' == uri_current[0]) {
+			/* Relative URI's beginning with '/' absolute-path based
+			 * at the root of the base uri
+			 */
+
+			separator = strchr (mutable_base_uri, ':');
+
+			/* g_assert (separator), really */
+			if (separator) {
+				/* If we start with //, skip past the authority section */
+				if ('/' == separator[1] && '/' == separator[2]) {
+					separator = strchr (separator + 3, '/');
+					if (separator) {
+						separator[0] = '\0';
+					}
+				} else {
+				/* If there's no //, just assume the scheme is the root */
+					separator[1] = '\0';
+				}
+			}
+		} else if ('#' != uri_current[0]) {
+			/* Handle the ".." convention for relative uri's */
+
+			/* If there's a trailing '/' on base_url, treat base_url
+			 * as a directory path.
+			 * Otherwise, treat it as a file path, and chop off the filename
+			 */
+
+			base_uri_length = strlen (mutable_base_uri);
+			if ('/' == mutable_base_uri[base_uri_length-1]) {
+				/* Trim off '/' for the operation below */
+				mutable_base_uri[base_uri_length-1] = 0;
+			} else {
+				separator = strrchr (mutable_base_uri, '/');
+				if (separator) {
+					*separator = '\0';
+				}
+			}
+
+			remove_internal_relative_components (uri_current);
+
+			/* handle the "../"'s at the beginning of the relative URI */
+			while (0 == strncmp ("../", uri_current, 3)) {
+				uri_current += 3;
+				separator = strrchr (mutable_base_uri, '/');
+				if (separator) {
+					*separator = '\0';
+				} else {
+					/* <shrug> */
+					break;
+				}
+			}
+
+			/* handle a ".." at the end */
+			if (uri_current[0] == '.' && uri_current[1] == '.' 
+			    && uri_current[2] == '\0') {
+
+			    	uri_current += 2;
+				separator = strrchr (mutable_base_uri, '/');
+				if (separator) {
+					*separator = '\0';
+				}
+			}
+
+			/* Re-append the '/' */
+			mutable_base_uri [strlen(mutable_base_uri)+1] = '\0';
+			mutable_base_uri [strlen(mutable_base_uri)] = '/';
+		}
+
+		result = g_strconcat (mutable_base_uri, uri_current, NULL);
+		g_free (mutable_base_uri); 
+		g_free (mutable_uri); 
+	}
+	
+	return result;
+}
+
+/**
+ * gnome_vfs_uri_list_parse:
+ * @uri_list:
+ * 
+ * Extracts a list of #GnomeVFSURI objects from a standard text/uri-list,
+ * such as one you would get on a drop operation.  Use
+ * #gnome_vfs_uri_list_free when you are done with the list.
+ *
+ * Return value: A GList of GnomeVFSURIs
+ **/
+GList*
+gnome_vfs_uri_list_parse (const gchar* uri_list)
+{
+	/* Note that this is mostly very stolen from old libgnome/gnome-mime.c */
+
+	const gchar *p, *q;
+	gchar *retval;
+	GnomeVFSURI *uri;
+	GList *result = NULL;
+
+	g_return_val_if_fail (uri_list != NULL, NULL);
+
+	p = uri_list;
+
+	/* We don't actually try to validate the URI according to RFC
+	 * 2396, or even check for allowed characters - we just ignore
+	 * comments and trim whitespace off the ends.  We also
+	 * allow LF delimination as well as the specified CRLF.
+	 */
+	while (p != NULL) {
+		if (*p != '#') {
+			while (g_ascii_isspace (*p))
+				p++;
+
+			q = p;
+			while ((*q != '\0')
+			       && (*q != '\n')
+			       && (*q != '\r'))
+				q++;
+
+			if (q > p) {
+				q--;
+				while (q > p
+				       && g_ascii_isspace (*q))
+					q--;
+
+				retval = g_malloc (q - p + 2);
+				strncpy (retval, p, q - p + 1);
+				retval[q - p + 1] = '\0';
+
+				uri = gnome_vfs_uri_new (retval);
+
+				g_free (retval);
+
+				if (uri != NULL)
+					result = g_list_prepend (result, uri);
+			}
+		}
+		p = strchr (p, '\n');
+		if (p != NULL)
+			p++;
+	}
+
+	return g_list_reverse (result);
+}
