@@ -31,56 +31,38 @@
    - Handle redirection.
    - Handle persistent connections.  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
+#include "http-method.h"
 
-#include <ctype.h>
-#include <string.h>
-#include <unistd.h>
-
-#include <glib.h>
-
-#include <stdlib.h> /* for atoi */
-
-#if GNOME_PLATFORM_VERSION < 1095000
-#include <gnome-xml/parser.h>
-#include <gnome-xml/tree.h>
-#include <gnome-xml/xmlmemory.h>
-#else
+#include "http-authn.h"
+#include "http-cache.h"
+/* Keep <sys/types.h> above any network includes for FreeBSD. */
+#include <sys/types.h>
+/* Keep <netinet/in.h> above <arpa/inet.h> for FreeBSD. */
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <gconf/gconf-client.h>
+#include <libgnomevfs/gnome-vfs-inet-connection.h>
+#include <libgnomevfs/gnome-vfs-mime-sniff-buffer.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-module-callback-module-api.h>
+#include <libgnomevfs/gnome-vfs-module.h>
+#include <libgnomevfs/gnome-vfs-private-utils.h>
+#include <libgnomevfs/gnome-vfs-socket-buffer.h>
+#include <libgnomevfs/gnome-vfs-socket.h>
+#include <libgnomevfs/gnome-vfs-ssl.h>
+#include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxml/xmlmemory.h>
-#endif
-
-#include <sys/time.h>
-
-
-#include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
-
 #include <pthread.h>
-
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 #include <stdarg.h>
 #include <stdio.h>
-
-#include "gnome-vfs.h"
-#include "gnome-vfs-private.h"
-#include "gnome-vfs-mime.h"
-#include "gnome-vfs-mime-sniff-buffer.h"
-#include "gnome-vfs-module.h"
-#include "gnome-vfs-module-callback-module-api.h"
-#include "gnome-vfs-standard-callbacks.h"
-#include "gnome-vfs-socket-buffer.h"
-#include "gnome-vfs-socket.h"
-#include "gnome-vfs-ssl.h"
-
-#include "http-method.h"
-#include "http-cache.h"
-#include "http-authn.h"
+#include <stdlib.h> /* for atoi */
+#include <string.h>
+#include <sys/socket.h>
+#include <sys/time.h>
+#include <unistd.h>
 
 #define EAZEL_XML_NS "http://services.eazel.com/namespaces"
 
@@ -331,7 +313,7 @@ parse_status (const char *cline,
 		
 		/* Calculate major HTTP version.  */
 		p = line;
-		for (mjr = 0; isdigit (*line); line++)
+		for (mjr = 0; g_ascii_isdigit (*line); line++)
 			mjr = 10 * mjr + (*line - '0');
 		if (*line != '.' || p == line)
 			return FALSE;
@@ -339,7 +321,7 @@ parse_status (const char *cline,
 		
 		/* Calculate minor HTTP version.  */
 		p = line;
-		for (mnr = 0; isdigit (*line); line++)
+		for (mnr = 0; g_ascii_isdigit (*line); line++)
 			mnr = 10 * mnr + (*line - '0');
 		if (*line != ' ' || p == line)
 			return -1;
@@ -361,7 +343,7 @@ parse_status (const char *cline,
 	}
 	
 	/* Calculate status code.  */
-	if (!(isdigit (*line) && isdigit (line[1]) && isdigit (line[2])))
+	if (!(g_ascii_isdigit (*line) && g_ascii_isdigit (line[1]) && g_ascii_isdigit (line[2])))
 		return -1;
 	statcode = 100 * (*line - '0') + 10 * (line[1] - '0') + (line[2] - '0');
 
@@ -426,7 +408,7 @@ header_value_to_number (const char *header_value,
 
 	p = header_value;
 
-	for (result = 0; isdigit ((unsigned char)*p); p++)
+	for (result = 0; g_ascii_isdigit (*p); p++)
 		result = 10 * result + (*p - '0');
 	if (*p)
 		return FALSE;
@@ -519,7 +501,7 @@ check_header (const char *header,
 	const char *p, *q;
 
 	for (p = header, q = name; *p != '\0' && *q != '\0'; p++, q++) {
-		if (tolower (*p) != tolower (*q))
+		if (g_ascii_tolower (*p) != g_ascii_tolower (*q))
 			break;
 	}
 
@@ -845,7 +827,7 @@ proxy_should_for_hostname (const char *hostname)
 	inet_aton("255.0.0.0", &in_mask); 
 
 	if (hostname != NULL 
-		&& (strcasecmp (hostname, "localhost") == 0 || (inet_aton (hostname, &in) != 0
+		&& (g_ascii_strcasecmp (hostname, "localhost") == 0 || (inet_aton (hostname, &in) != 0
 			&& ((in.s_addr & in_mask.s_addr) == in_loop.s_addr)))) {
 		ret = FALSE;
 	}
@@ -1073,7 +1055,7 @@ connect_to_uri (
 	g_return_val_if_fail (p_proxy_connect != NULL, GNOME_VFS_ERROR_INTERNAL);
 	g_return_val_if_fail (toplevel_uri != NULL, GNOME_VFS_ERROR_INTERNAL);
 
-	if (!strcasecmp (gnome_vfs_uri_get_scheme (&toplevel_uri->uri), 
+	if (!g_ascii_strcasecmp (gnome_vfs_uri_get_scheme (&toplevel_uri->uri), 
 				"https")) {
 		if (!gnome_vfs_ssl_enabled ()) {
 			return GNOME_VFS_ERROR_NOT_SUPPORTED;
@@ -1192,8 +1174,8 @@ build_request (const char * method, GnomeVFSToplevelURI * toplevel_uri, gboolean
 	/* Request line.  */
 	request = g_string_new ("");
 
-	g_string_sprintfa (request, "%s %s%s HTTP/1.0\r\n", method, uri_string,
-		strlen(gnome_vfs_uri_get_path (uri)) == 0 ? "/" : "" );
+	g_string_append_printf (request, "%s %s%s HTTP/1.0\r\n", method, uri_string,
+				gnome_vfs_uri_get_path (uri)[0] == '\0' ? "/" : "" );
 
 	DEBUG_HTTP (("-->Making request '%s %s'", method, uri_string));
 	
@@ -1202,11 +1184,11 @@ build_request (const char * method, GnomeVFSToplevelURI * toplevel_uri, gboolean
 
 	/* `Host:' header.  */
 	if(toplevel_uri->host_port && toplevel_uri->host_port != 0) {
-		g_string_sprintfa (request, "Host: %s:%d\r\n",
-			   toplevel_uri->host_name, toplevel_uri->host_port);
+		g_string_append_printf (request, "Host: %s:%d\r\n",
+					toplevel_uri->host_name, toplevel_uri->host_port);
 	} else {
-		g_string_sprintfa (request, "Host: %s:80\r\n",
-			   toplevel_uri->host_name);
+		g_string_append_printf (request, "Host: %s:80\r\n",
+					toplevel_uri->host_name);
 	}
 
 	/* `Accept:' header.  */
@@ -1219,7 +1201,7 @@ build_request (const char * method, GnomeVFSToplevelURI * toplevel_uri, gboolean
 		user_agent = USER_AGENT_STRING;
 	}
 
-	g_string_sprintfa (request, "User-Agent: %s\r\n", user_agent);
+	g_string_append_printf (request, "User-Agent: %s\r\n", user_agent);
 
 	return request;
 }
@@ -1318,12 +1300,12 @@ make_request (HttpFileHandle **handle_return,
 		
 		/* `Content-Length' header.  */
 		if (data != NULL) {
-			g_string_sprintfa (request, "Content-Length: %d\r\n", data->len);
+			g_string_append_printf (request, "Content-Length: %d\r\n", data->len);
 		}
 		
 		/* Extra headers. */
 		if (extra_headers != NULL) {
-			g_string_append(request, extra_headers);
+			g_string_append (request, extra_headers);
 		}
 
 		/* Empty line ends header section.  */
@@ -1684,7 +1666,7 @@ process_propfind_propstat (xmlNodePtr node,
 				} else if (strcmp ((char *)l->name, "nautilus-treat-as-directory") == 0
 					   && l->ns != NULL && l->ns->href != NULL
 					   && strcmp (l->ns->href, EAZEL_XML_NS) == 0
-					   && strcasecmp (node_content_xml, "TRUE") == 0) {
+					   && g_ascii_strcasecmp (node_content_xml, "TRUE") == 0) {
 					treat_as_directory = TRUE;
 				}
 				/* Unfortunately, we don't have a mapping for "creationdate" */
@@ -1776,22 +1758,6 @@ null_handling_strcmp (const char *a, const char *b)
 }
 
 #if 0
-static unsigned char
-unhex_char (char to_unhex)
-{
-	unsigned char ret = 0;
-	
-	if (to_unhex >= 'A' && to_unhex <= 'F') {	
-		ret = to_unhex - 'A' + 0xA;	
-	} else if (to_unhex >= 'a' && to_unhex <= 'f') {
-		ret = to_unhex - 'a' + 0xa;	
-	} else if (to_unhex >= '0' && to_unhex <= '9') {
-		ret = to_unhex - '0';
-	}
-	
-	return ret;
-}
-
 static char *
 unescape_unreserved_chars (const char *in_string)
 {
@@ -1809,13 +1775,13 @@ unescape_unreserved_chars (const char *in_string)
 
 	for (read_char = in_string, write_char = ret ; *read_char != '\0' ; read_char++) {
 		if (read_char[0] == '%' 
-		    && isxdigit ((unsigned char) read_char[1]) 
-		    && isxdigit ((unsigned char) read_char[2])) {
+		    && g_ascii_isxdigit (read_char[1]) 
+		    && g_ascii_isxdigit (read_char[2])) {
 			char unescaped;
 			
-			unescaped = (unsigned char) ((unhex_char (read_char[1]) << 4) | unhex_char (read_char[2]));
+			unescaped = (g_ascii_xdigit_value (read_char[1]) << 4) | g_ascii_xdigit_value (read_char[2]);
 			
-			if (strchr (reserved_chars, (unsigned char)unescaped)) {
+			if (strchr (reserved_chars, unescaped)) {
 				*write_char++ = *read_char++;
 				*write_char++ = *read_char++;
 				*write_char++ = *read_char; /*The last ++ is done in the for statement */
@@ -1836,7 +1802,6 @@ unescape_unreserved_chars (const char *in_string)
 	
 	return ret;
 }
-
 #endif /* 0 */
 
 static xmlNodePtr
@@ -2094,7 +2059,6 @@ do_open_directory(GnomeVFSMethod *method,
 		  GnomeVFSMethodHandle **method_handle,
 		  GnomeVFSURI *uri,
 		  GnomeVFSFileInfoOptions options,
-		  const GnomeVFSDirectoryFilter *filter,
 		  GnomeVFSContext *context) 
 {
 	/* TODO move to using the gnome_vfs_file_info_list family of functions */
@@ -2104,7 +2068,7 @@ do_open_directory(GnomeVFSMethod *method,
 	GList *child_file_info_cached_list = NULL;
 
 	ANALYZE_HTTP ("==> +do_open_directory");
-	DEBUG_HTTP (("+Open_Directory options: %d dirfilter: 0x%08x URI: '%s'", options, (unsigned int) filter, gnome_vfs_uri_to_string( uri, 0)));
+	DEBUG_HTTP (("+Open_Directory options: %d URI: '%s'", options, gnome_vfs_uri_to_string (uri, 0)));
 
 	/* Check the cache--is this even a directory?  
 	 * (Nautilus, in particular, seems to like to make this call on non directories
@@ -2141,7 +2105,7 @@ do_open_directory(GnomeVFSMethod *method,
 			if (uri->text != NULL && *uri->text != '\0'
 			   && uri->text[strlen (uri->text) - 1] != '/') {
 				GnomeVFSURI *tmpuri = gnome_vfs_uri_append_path (uri, "/");
-				result = do_open_directory (method, (GnomeVFSMethodHandle **)&handle, tmpuri, options, filter, context);
+				result = do_open_directory (method, (GnomeVFSMethodHandle **)&handle, tmpuri, options, context);
 				gnome_vfs_uri_unref (tmpuri);
 
 			}
@@ -2589,23 +2553,11 @@ vfs_module_init (const char *method_name,
 		gconf_init (argc, argv, NULL);
 	}
 
-	/* ensure GTK is inited for gconf-client. */
-#if GNOME_PLATFORM_VERSION < 1095000
-	gtk_type_init ();
-#else
-	gtk_type_init (G_TYPE_DEBUG_NONE);
-#endif
-
 	gl_client = gconf_client_get_default ();
 
-#if GNOME_PLATFORM_VERSION < 1095000
-	gtk_object_ref (GTK_OBJECT (gl_client));
-	gtk_object_sink (GTK_OBJECT (gl_client));
-#endif
-
 	gl_mutex = g_mutex_new ();
 	
-	gconf_client_add_dir (gl_client, PATH_GCONF_GNOME_VFS, GCONF_CLIENT_PRELOAD_NONE, &gconf_error);
+	gconf_client_add_dir (gl_client, PATH_GCONF_GNOME_VFS, GCONF_CLIENT_PRELOAD_ONELEVEL, &gconf_error);
 
 	if (gconf_error) {
 		DEBUG_HTTP (("GConf error during client_add_dir '%s'", gconf_error->message));
@@ -2613,10 +2565,6 @@ vfs_module_init (const char *method_name,
 		gconf_error = NULL;
 	}
 
-#if GNOME_PLATFORM_VERSION < 1095000
-	gtk_signal_connect (GTK_OBJECT (gl_client), "value_changed", (GtkSignalFunc) sig_gconf_value_changed, NULL);
-#endif
-
 	/* Load the http proxy setting */	
 	proxy_value = gconf_client_get (gl_client, KEY_GCONF_USE_HTTP_PROXY, &gconf_error);
 
@@ -2649,16 +2597,7 @@ vfs_module_init (const char *method_name,
 void
 vfs_module_shutdown (GnomeVFSMethod *method)
 {
-#if GNOME_PLATFORM_VERSION < 1095000
-	gtk_signal_disconnect_by_func (GTK_OBJECT(gl_client), (GtkSignalFunc) sig_gconf_value_changed, NULL);
-#endif
-
-#if GNOME_PLATFORM_VERSION < 1095000
-	gtk_object_destroy (GTK_OBJECT (gl_client));
-	gtk_object_unref (GTK_OBJECT (gl_client));
-#else
 	g_object_unref (G_OBJECT (gl_client));
-#endif
 
 	http_authn_shutdown ();
 	
